# -*- coding: utf-8 -*-
##############################################################################
import wizard
import time
import datetime
import pooler
import base64
from osv import fields, osv
from tools.translate import _
from pyExcelerator import *

form = """<?xml version="1.0"?>
<form string="Yearly VAT Concordance">
    <field name="period_from"/>
    <field name="period_to"/>
    <newline/>
    <field name="accounts_list" colspan="4"/>
    <field name="vat_cases_list" colspan="4"/>
    <field name="sales_journals" colspan="4"/>
</form>"""

fields = {
    'accounts_list': {'string':'Accounts','type':'char','size':250,'required':True,'default':'6,7','help':'List of accounts separated by commas and you can give only the beginning of all accounts codes'},
    'vat_cases_list': {'string':'Taxes Codes','type':'char','size':250,'required':True,'default':'00,01,02,03,44,45,46,47,48,49','help':'List of taxes codes numbers separated by commas'},
    'sales_journals': {'string':'Sales Journals','type':'char','size':'250','required':True,'default':'VEN,NCV'},
    'period_from': {'string':'From Period','type':'many2one','relation':'account.period','required':True},
    'period_to': {'string':'To Period','type':'many2one','relation':'account.period','required':True},
    }

msg_form = """<?xml version="1.0"?>
<form string="Notification">
    <separator string="XLS File has been Created."  colspan="4"/>
    <field name="vat_concordance_yearly.xls" colspan="4"/>
</form>"""
msg_fields = {
  'vat_concordance_yearly.xls':{'string': 'Save XLS File','type': 'binary','readonly': True,},
}

def num_to_letter(line_number_in_pyexcel):
    # this method convert an integer to a line label in Excel
    # first line = 0 (beware on pyExcelerator, first line = 0)
    # 0 => 'A'
    # 26 => 'AA'
    # 27 => 'AB'
    line_number_in_pyexcel += 1
    if line_number_in_pyexcel < 26:
        result = chr(64+line_number_in_pyexcel)
    else:
        first = line_number_in_pyexcel_number_in_pyexcel // 26
        last = number % 26
        result = chr(64+first)+chr(64+last)
    return result
def full_cell(line_number_in_pyexcel,col_number_in_pyexcel,fixed='none'):
    # fixed = none, col, line, both
    result = ''
    if fixed in ('col','both'):
        result += '$'
    result += num_to_letter(col_number_in_pyexcel)
    if fixed in ('line','both'):
        result += '$'
    result += str(line_number_in_pyexcel+1)
    return result

class vat_concordance_yearly(wizard.interface):

    def _create_xls(self, cr, uid, data, context):
        pool = pooler.get_pool(cr.dbname)

        period_obj = pool.get('account.period')
        account_obj = pool.get('account.account')
        tax_code_obj = pool.get('account.tax.code')
        journal_obj = pool.get('account.journal')
        account_move_line_obj = pool.get('account.move.line')
        #
        from_period = period_obj.browse(cr,uid,data['form']['period_from'])
        to_period = period_obj.browse(cr,uid,data['form']['period_to'])
        period_ids = period_obj.search(cr,uid,[('date_start','>=',from_period.date_start),('date_stop','<=',to_period.date_stop),('special','=',False)])
        periods = period_obj.read(cr,uid,period_ids,['name'])
        dPeriods = {}
        for per in periods:
            dPeriods[per['id']] = per
        #
        laccounts = data['form']['accounts_list'].split(',')
        laccounts = [x.strip() for x in laccounts]
        selected_account_ids = []
        for account_code_beginning in laccounts:
            account_ids = account_obj.search(cr,uid,[('code','ilike',account_code_beginning)])
            if account_ids:
                selected_account_ids.extend(account_ids)
        #
        lcases = data['form']['vat_cases_list'].split(',')
        lcases = [x.strip() for x in lcases]
        selected_tax_code_ids = []
        for tax_case in lcases:
            tax_code_ids = tax_code_obj.search(cr,uid,[('code','=',tax_case)])
            if tax_code_ids:
                selected_tax_code_ids.extend(tax_code_ids)
        tax_codes = tax_code_obj.read(cr,uid,selected_tax_code_ids,['id','code','name','sign'])
        dTaxCodes = {}
        for tax_code in tax_codes:
            dTaxCodes[tax_code['id']] = tax_code
        #
        ljournals = data['form']['sales_journals'].split(',')
        ljournals = [x.strip() for x in ljournals]
        journal_ids = journal_obj.search(cr,uid,[('code','in',ljournals)])
        journals = journal_obj.read(cr,uid,journal_ids,['code','name','refund_journal'])
        dJournals = {}
        for jnl in journals:
            dJournals[jnl['id']] = jnl
        #
        wb = Workbook()
        ws1 = wb.add_sheet(_('Main Result'))
        ws1.write(0,0,_('VAT Concordance Yearly'))
        ws1.write(1,0,'Date:')
        ws1.write(1,1,datetime.date.today().strftime('%d/%m/%Y'))
        ws1.write(2,0,_('From'))
        ws1.write(2,1,from_period.name)
        ws1.write(3,0,_('To'))
        ws1.write(3,1,to_period.name)
        ws1.write(4,0,_('Accounts List'))
        ws1.write(4,1,data['form']['accounts_list'])
        ws1.write(5,0,_('Taxes Codes List'))
        ws1.write(5,1,data['form']['vat_cases_list'])
        ws1.write(6,0,_('Journals'))
        ws1.write(6,1,data['form']['sales_journals'])
        ws1.write(7,0,_('This sheets sums by period and tax code ALL moves in Sales Journals AND Accounts\'List. The other moves are in ERRORS sheets even if it\'s not really an error.'))
        ws1.write(10,0,_('Codes'))
        dTaxCodeID2Col = {}
        col = 1
        for tax_code_id in selected_tax_code_ids:
            ws1.write(9,col,dTaxCodes[tax_code_id]['code'])
            ws1.write(10,col,dTaxCodes[tax_code_id]['name'])
            dTaxCodeID2Col[tax_code_id] = col
            col += 1
        ws1.write(10,col,_('Total'))
        
        dPeriodID2Line = {}
        first_line = 11
        line = first_line
        for period_id in period_ids:
            ws1.write(line,0,dPeriods[period_id]['name'])
            ws1.write(line,col,Formula("SUM(%s:%s)" % (full_cell(line,1),full_cell(line,col-1))))
            dPeriodID2Line[period_id] = line
            line += 1
        ws1.write(line,0,_('Total'))
        col_total = 1
        while col_total <= col:
            ws1.write(line,col_total,Formula("SUM(%s:%s)" % (full_cell(first_line,col_total),full_cell(line-1,col_total))))
            col_total += 1
        #
        ws2 = wb.add_sheet(_('Case NC but not NCV'))
        ws2.write(0,0,_('VAT Concordance Yearly - Detected Errors Type 1'))
        ws2.write(1,0,'Date:')
        ws2.write(1,1,datetime.date.today().strftime('%d/%m/%Y'))
        ws2.write(3,1,_('This sheet lists move lines on case 48 or 49 but not on NCV journal'))
        ws2.write(5,0,_('line ID'))
        ws2.write(5,1,_('VAT Code'))
        ws2.write(5,2,_('Account'))
        ws2.write(5,3,_('Journal'))
        ws2.write(5,4,_('Move'))
        ws2.write(5,5,_('Tax Amount'))
        ws2.write(5,6,_('Line Details'))
        ws2_line = 6
        # sum by period and case all lines in the periods and accounts
        amline_ids = account_move_line_obj.search(cr,uid,[('period_id','in',period_ids),('tax_code_id','in',selected_tax_code_ids),('journal_id','in',journal_ids),('account_id','in',selected_account_ids)])
        if amline_ids:
            dTots = {}
            index = 0
            size = 500
            while amline_ids[index:index+size]:
                amlines = account_move_line_obj.browse(cr,uid,amline_ids[index:index+size])
                for amline in amlines:
                    # the only 'errors' we can find here are 'refund journal' moves on not 'refund tax code' based on 'sign' field
                    if (amline.journal_id.refund_journal and amline.tax_code_id.sign == 1) or ((not amline.journal_id.refund_journal) and (amline.tax_code_id.sign == -1)):
                        #print 'strange move on strange tax code'
                        ws2.write(ws2_line,0,amline.id)
                        ws2.write(ws2_line,1,amline.tax_code_id.code)
                        ws2.write(ws2_line,2,amline.account_id.code)
                        ws2.write(ws2_line,2,amline.journal_id.code)
                        ws2.write(ws2_line,3,amline.move_id.ref or amline.move_id.name or '')
                        ws2.write(ws2_line,4,amline.tax_amount or 0.0)
                        ws2.write(ws2_line,5,amline.name or '')
                        ws2_line += 1
                    key = (amline.period_id.id,amline.tax_code_id.id)
                    if dTots.has_key(key):
                        new_value = dTots[key]
                    else:
                        new_value = 0.0
                    new_value = new_value + (amline.tax_amount * amline.tax_code_id.sign)
                    dTots[key] = new_value
                index += size
            for (key,value) in dTots.items():
                ws1.write(dPeriodID2Line[key[0]],dTaxCodeID2Col[key[1]],value)
        #
        ws3 = wb.add_sheet(_('Move lines on tax codes but not sales journals'))
        ws3.write(0,0,_('VAT Concordance Yearly - Detected Errors Type 2'))
        ws3.write(1,0,'Date:')
        ws3.write(1,1,datetime.date.today().strftime('%d/%m/%Y'))
        ws3.write(2,0,_('Taxes Codes List'))
        ws3.write(2,1,data['form']['vat_cases_list'])
        ws3.write(3,0,_('Journals'))
        ws3.write(3,1,data['form']['sales_journals'])
        ws3.write(5,1,_('This sheet lists move lines on selected cases but not in sales journals'))
        ws3.write(7,0,_('line ID'))
        ws3.write(7,1,_('VAT Code'))
        ws3.write(7,2,_('Account'))
        ws3.write(7,3,_('Journal'))
        ws3.write(7,4,_('Move'))
        ws3.write(7,5,_('Tax Amount'))
        ws3.write(7,6,_('Line Details'))
        ws3_line = 8
        amline_ids = account_move_line_obj.search(cr,uid,[('period_id','in',period_ids),('tax_code_id','in',selected_tax_code_ids),('journal_id','not in',journal_ids),('account_id','in',selected_account_ids)])
        if amline_ids:
            amlines = account_move_line_obj.browse(cr,uid,amline_ids)
            for amline in amlines:
                ws3.write(ws3_line,0,amline.id)
                ws3.write(ws3_line,1,amline.tax_code_id.code)
                ws3.write(ws3_line,2,amline.account_id.code)
                ws3.write(ws3_line,3,amline.journal_id.code)
                ws3.write(ws3_line,4,amline.move_id.name or amline.move_id.ref or '')
                ws3.write(ws3_line,5,amline.tax_amount or 0.0)
                ws3.write(ws3_line,6,amline.name or '')
                ws3_line += 1
        #
        ws4 = wb.add_sheet(_('Move lines on accounts but not sales journals'))
        ws4.write(0,0,_('VAT Concordance Yearly - Detected Errors Type 3'))
        ws4.write(1,0,'Date:')
        ws4.write(1,1,datetime.date.today().strftime('%d/%m/%Y'))
        ws4.write(2,0,_('Accounts'))
        ws4.write(2,1,data['form']['accounts_list'])
        ws4.write(3,0,_('Journals'))
        ws4.write(3,1,data['form']['sales_journals'])
        ws4.write(5,1,_('This sheet lists move lines on selected accounts but not in sales journals'))
        ws4.write(7,0,_('line ID'))
        ws4.write(7,1,_('VAT Code'))
        ws4.write(7,2,_('Account'))
        ws4.write(7,3,_('Journal'))
        ws4.write(7,4,_('Move'))
        ws4.write(7,5,_('Tax Amount'))
        ws4.write(7,6,_('Line Details'))
        ws4_line = 8
        amline_ids = account_move_line_obj.search(cr,uid,[('period_id','in',period_ids),('journal_id','not in',journal_ids),('account_id','in',selected_account_ids)])
        if amline_ids:
            amlines = account_move_line_obj.browse(cr,uid,amline_ids)
            for amline in amlines:
                print amline
                print amline.account_id
                print amline.account_id.code
                ws4.write(ws4_line,0,amline.id)
                ws4.write(ws4_line,1,amline.tax_code_id and amline.tax_code_id.code or '')
                ws4.write(ws4_line,2,amline.account_id.code)
                ws4.write(ws4_line,3,amline.journal_id.code)
                ws4.write(ws4_line,4,amline.move_id.name or amline.move_id.ref or '')
                ws4.write(ws4_line,5,amline.tax_amount or 0.0)
                ws4.write(ws4_line,6,amline.name or '')
                ws4_line += 1
        #
        ws5 = wb.add_sheet(_('Move lines on sales journal but on other accounts'))
        ws5.write(0,0,_('VAT Concordance Yearly - Detected Errors Type 4'))
        ws5.write(1,0,'Date:')
        ws5.write(1,1,datetime.date.today().strftime('%d/%m/%Y'))
        ws5.write(2,0,_('Accounts'))
        ws5.write(2,1,data['form']['accounts_list'])
        ws5.write(3,0,_('Journals'))
        ws5.write(3,1,data['form']['sales_journals'])
        ws5.write(5,1,_('This sheet lists move lines on selected accounts but not in sales journals'))
        ws5.write(7,0,_('line ID'))
        ws5.write(7,1,_('VAT Code'))
        ws5.write(7,2,_('Account'))
        ws5.write(7,3,_('Journal'))
        ws5.write(7,4,_('Move'))
        ws5.write(7,5,_('Tax Amount'))
        ws5.write(7,6,_('Line Details'))
        ws5_line = 8
        amline_ids = account_move_line_obj.search(cr,uid,[('period_id','in',period_ids),('journal_id','in',journal_ids),('account_id','not in',selected_account_ids)])
        if amline_ids:
            amlines = account_move_line_obj.browse(cr,uid,amline_ids)
            for amline in amlines:
                ws5.write(ws5_line,0,amline.id)
                ws5.write(ws5_line,1,amline.tax_code_id and amline.tax_code_id.code or '')
                ws5.write(ws5_line,2,amline.account_id.code)
                ws5.write(ws5_line,3,amline.journal_id.code)
                ws5.write(ws5_line,4,amline.move_id.name or amline.move_id.ref or '')
                ws5.write(ws5_line,5,amline.tax_amount or 0.0)
                ws5.write(ws5_line,6,amline.name or '')
                ws5_line += 1
        wb.save('vat_concordance_yearly.xls')
        result_file = open('vat_concordance_yearly.xls','rb').read()

        # give the result tos the user
        data['form']['vat_concordance_yearly.xls']=base64.encodestring(result_file)
        return data['form']

    states = {
        'init': {
            'actions': [],
            'result': {'type': 'form', 'arch':form, 'fields': fields, 'state':[('end','Cancel'),('go','Create XLS') ]}
                },
         'go': {
            'actions': [_create_xls],
            'result': {'type':'form', 'arch':msg_form, 'fields':msg_fields, 'state':[('end','Ok')]},
                }
             }
vat_concordance_yearly('vat_concordance_yearly')
