# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2008 CCI Connect ASBL. (http://www.cciconnect.be) All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################
import datetime
from osv import fields, osv
from tools.translate import _
import tools

#SUBJECT_MSG_JOB = u"Confirmation départ de %s de l'entreprise %s ?"
#MSG_JOB = u"""
#<p>En validant les modifications apportées par le membre à sa propre fiche, via le site Web, l'ERP a constaté qu'il a tenté d'effacer %s de son personnel alors qu'il/elle %s '%s' dont vous êtes responsable.</p>
#<p>Cet 'effacement' a été annulé et juste remplacé par une simple absence du répertoire des membres papier et une exclusion du répertoire de l'app mobile CCI.</p>
#<p>Mais il faudrait peut-être vérifier si ce départ de l'entreprise n'est pas réel.</p>
#<p>Si tel est le cas, merci de bien effacer vous-même manuellement la présence de %s dans la fiche de l'entreprise %s.</p>
#<p>Merci d'avance.</p>
#<p>L'ERP...</p>
#"""

class cci_popup_stand_type(osv.osv):
    _name = "cci.popup.stand_type"
    _description = "type of stand for popup"
    _columns = {
        'name': fields.char('Name',size=30,required=True),
        'default_price': fields.float('Default Price',digits=(15,2)),
        'default_exhibitors': fields.integer('Default Free Exhibitors'),
        'default_guests': fields.integer('Default Free Guests'),
        'default_champagne': fields.integer('Default Champagne Bottles'),
        'default_night_tickets': fields.integer('Default Night Tickets'),
        'web_type': fields.integer('Type on Popup Plateform',required=True),
        'event_id': fields.many2one('event.event','Event for Registrations'),
        'product_id': fields.many2one('product.product','Product to Invoice stand'),
        'night_product_id': fields.many2one('product.product','Product to Invoice Night Bracelets'),
        'exhibitor_product_id': fields.many2one('product.product','Product to Invoice Additional Exhibitor Tickets'),
        'guest_product_id': fields.many2one('product.product','Product to Invoice Additional Guest Tickets'),
        'active': fields.boolean('Active'),
    }
    _defaults = {
        'active': lambda *a: True,
    }
cci_popup_stand_type()

class cci_popup_stand(osv.osv):
    _name = "cci.popup.stand"
    _description = "folder to manage a stand for an edition of popup"
    def _get_salesman(self, cr, uid, ids, field_name, arg, context):
        res = {}
        for stand in self.browse(cr,uid,ids):
            if stand.partner_id and stand.partner_id.user_id:
                res[stand.id] = stand.partner_id.user_id.name + ' [' + stand.partner_id.user_id.login +']'
            else:
                res[stand.id] = ''
        return res
    def _get_invoices_amount(self, cr, uid, ids, field_name, arg, context):
        res = {}
        for stand in self.browse(cr,uid,ids):
            res[stand.id] = 0.0
            multiplier = 1
            if stand.invoice_id and stand.invoice_id.journal_id.refund_journal:
                multiplier = -1
            res[stand.id] += ( stand.invoice_id.amount_untaxed * multiplier )
        return res
    def _get_email_order(self, cr, uid, ids, field_name, arg, context):
        res = {}
        for stand in self.browse(cr,uid,ids):
            email = ''
            if stand.partner_id and stand.order_contact_id:
                if stand.partner_id.address:
                    for addr in stand.partner_id.address:
                        if addr.job_ids:
                            for job in addr.job_ids:
                                if job.contact_id and job.contact_id.id == stand.order_contact_id.id:
                                    email = job.email or job.contact_id.email or ''
                                    if email:
                                        break
                        if email:
                            break
            if not email:
                if stand.order_contact_id.email:
                    email = stand.order_contact_id.email
            res[stand.id] = email
        return res
    def _get_email_material(self, cr, uid, ids, field_name, arg, context):
        res = {}
        for stand in self.browse(cr,uid,ids):
            email = ''
            if stand.partner_id and stand.material_contact_id:
                if stand.partner_id.address:
                    for addr in stand.partner_id.address:
                        if addr.job_ids:
                            for job in addr.job_ids:
                                if job.contact_id and job.contact_id.id == stand.material_contact_id.id:
                                    email = job.email or job.contact_id.email or ''
                                    if email:
                                        break
                        if email:
                            break
            if not email:
                if stand.material_contact_id.email:
                    email = stand.material_contact_id.email
            res[stand.id] = email
        return res
    def _get_email_stand(self, cr, uid, ids, field_name, arg, context):
        res = {}
        for stand in self.browse(cr,uid,ids):
            email = ''
            if stand.partner_id and stand.stand_contact_id:
                if stand.partner_id.address:
                    for addr in stand.partner_id.address:
                        if addr.job_ids:
                            for job in addr.job_ids:
                                if job.contact_id and job.contact_id.id == stand.stand_contact_id.id:
                                    email = job.email or job.contact_id.email or ''
                                    if email:
                                        break
                        if email:
                            break
            if not email:
                if stand.stand_contact_id.email:
                    email = stand.stand_contact_id.email
            res[stand.id] = email
        return res
    def onchange_type_id(self, cr, uid, ids, type_id):
        if type_id:
            stand_type = self.pool.get('cci.popup.stand_type').browse(cr, uid, type_id)
            if stand_type.default_price:
                result = {'value': {'final_stand_price': stand_type.default_price}}
            else:
                result = {'value': {'final_stand_price': 0.0}}
            if stand_type.default_night_tickets:
                result['value']['order_night'] = stand_type.default_night_tickets
                result['value']['price_night'] = 0.0
            else:
                result['value']['order_night'] = 0
                result['value']['price_night'] = 20.0
        else:
            result = {'value':{'final_stand_price':0.0,'order_night':0,'price_night':20.0}}
        return result
    def onchange_partner_id(self, cr, uid, ids, partner_id):
        data={}
        #raise an error if the partner has a warning
        warning = False
        if partner_id:
            data_partner = self.pool.get('res.partner').browse(cr,uid,partner_id)
            if data_partner.alert_events:
                warning = {
                    'title': "Warning:",
                    'message': data_partner.alert_explanation or 'Partner is not valid but there is no explanation on partner, only the \'event warning\' selected'
                        }
        #data = {'value':data}
        if warning:
            data['warning'] =  warning
        return data
    def onchange_order_contact_id(self, cr, uid, ids, contact_id, stand_contact_id, material_contact_id):
        if contact_id and ((not stand_contact_id) or (not material_contact_id)):
            new_contacts = {}
            if not stand_contact_id:
                new_contacts['stand_contact_id'] = contact_id
            if not material_contact_id:
                new_contacts['material_contact_id'] = contact_id
            if new_contacts:
                result = {'value': new_contacts}
            else:
                result = {'value':{}}
        else:
            result = {'value':{}}
        return result
    def create(self,cr,uid,vals,context=None):
        if vals and vals.has_key('type_id'):
            stand_type = self.pool.get('cci.popup.stand_type').browse(cr,uid,vals['type_id'])
            if stand_type.event_id:
                contact_name = 'unknown'
                if vals['order_contact_id']:
                    contact = self.pool.get('res.partner.contact').browse(cr,uid,vals['order_contact_id'])
                    contact_name = (contact.name+' '+(contact.first_name or '')).strip()
                reg_obj = self.pool.get('event.registration')
                reg_data = {'event_id':stand_type.event_id.id,
                            'partner_invoice_id':vals['partner_id'],
                            'partner_id':vals['partner_id'],
                            'contact_id':vals['order_contact_id'],
                            'unit_price':vals['final_stand_price'],
                            'tobe_invoiced':False,
                            'attendance':True,
                            'nb_register':1,
                            'name':'Registration: '+contact_name,
                            'invoice_label': stand_type.event_id.product_id and stand_type.event_id.product_id.name or 'Popup',
                           }
                reg_id = reg_obj.create(cr,uid,reg_data,context)
                if reg_id:
                    new_reg_data = {'state':'done',}
                    new_values = reg_obj.onchange_contact_id(cr,uid,[reg_id,], vals['order_contact_id'], vals['partner_id'])
                    new_reg_data['badge_name'] = new_values.has_key('badge_name') and new_values['value']['badge_name'] or ''
                    new_reg_data['email_from'] = new_values.has_key('email_from') and new_values['value']['email_from'] or ''
                    new_reg_data['badge_title'] = new_values.has_key('badge_title') and new_values['value']['badge_title'] or ''
                    new_values = reg_obj.onchange_partner_id(cr,uid,[reg_id,], vals['partner_id'],stand_type.event_id.id)
                    new_reg_data['badge_partner'] = new_values.has_key('badge_partner') and new_values['value']['badge_partner'] or ''
                    reg_obj.write(cr,uid,[reg_id,],new_reg_data)
                    vals['stand_reg_id'] = reg_id
        result = super(cci_popup_stand, self).create(cr, uid, vals, context)
        return result
    def write(self,cr,uid,ids,vals,context=None):
        for stand_id in ids:
            if ('type_id' in vals) or ('partner_id' in vals) or ('order_contact_id' in vals) or ('final_stand_price' in vals):
                # changes to write down to linked event.registration
                stand = self.browse(cr,uid,stand_id)
                if stand.stand_reg_id:
                    reg_data = {}
                    if 'type_id' in vals:
                        stand_type = self.pool.get('cci.popup.stand_type').browse(cr,uid,vals['type_id'])
                        if stand_type.event_id:
                            reg_data['event_id'] = stand_type.event_id.id
                            reg_data['invoice_label'] = stand_type.event_id.product_id and stand_type.event_id.product_id.name or 'Popup'
                    if 'partner_id' in vals:
                        reg_data['partner_invoice_id'] = vals['partner_id']
                        reg_data['partner_id'] = vals['partner_id']
                        event_id = 0
                        if 'type_id' in vals:
                            stand_type = self.pool.get('cci.popup.stand_type').browse(cr,uid,vals['type_id'])
                            event_id = stand_type.event_id.id
                        else:
                            event_id = stand.type_id.event_id.id
                        new_values = self.pool.get('event.registration').onchange_partner_id(cr,uid,[stand.stand_reg_id,],vals['partner_id'],event_id)
                        reg_data['badge_partner'] = new_values.has_key('badge_partner') and new_values['value']['badge_partner'] or ''
                    if 'order_contact_id' in vals:
                        contact = self.pool.get('res.partner.contact').browse(cr,uid,vals['order_contact_id'])
                        contact_name = (contact.name+' '+(contact.first_name or '')).strip()
                        reg_data['contact_id'] = vals['order_contact_id']
                        reg_data['name'] = 'Registration: '+contact_name
                        if 'partner_id' in vals:
                            partner_id = vals['partner_id']
                        else:
                            partner_id = stand.partner_id.id
                        new_values = self.pool.get('event.registration').onchange_contact_id(cr,uid,[stand.stand_reg_id,], vals['order_contact_id'], partner_id)
                        reg_data['badge_name'] = new_values.has_key('badge_name') and new_values['value']['badge_name'] or ''
                        reg_data['email_from'] = new_values.has_key('email_from') and new_values['value']['email_from'] or ''
                        reg_data['badge_title'] = new_values.has_key('badge_title') and new_values['value']['badge_title'] or ''
                    if 'final_stand_price' in vals:
                        reg_data['unit_price'] = vals['final_stand_price']
                    self.pool.get('event.registration').write(cr,uid,[stand.stand_reg_id.id,],reg_data,context)
        return super(cci_popup_stand,self).write(cr,uid,ids,vals,context)
    def copy(self, cr, uid, id, default=None, context={}):
        stand = self.read(cr,uid,id,['name'])
        if not default:
            default = {}
        default.update({
            'name':(stand['name'] and stand['name'].strip() or '')+' (copy)',
            'edition':False,
            'invoice_id': False,
            'stand_reg_id':False,
            'external_name':False,
            'video_status':'waiting',
            'canvas_status':'waiting',
            'web_id':False,
            'confirm_sending':False,
        })
        return super(cci_popup_stand, self).copy(cr, uid, id, default, context)
    _columns = {
        'edition': fields.integer('Edition',required=True),
        'type_id': fields.many2one('cci.popup.stand_type','Type',required=True),
        'partner_id': fields.many2one('res.partner','Partner',required=True),
        'salesman': fields.function(_get_salesman,type='string',size=254,method=True,string='Salesman'),
        'order_contact_id': fields.many2one('res.partner.contact','Order Contact'),
        'stand_contact_id': fields.many2one('res.partner.contact','Stand Contact'),
        'name': fields.char('Stand Name',size=90),
        'final_stand_price': fields.float('Stand Price',digits=(15,2)),
        'order_night': fields.integer('Entries for the night'),
        'price_night': fields.float('Night entries Price',digits=(15,2)),
        'order_add_exhibitor': fields.integer('Additional Exhibitors'),
        'price_add_exhibitor': fields.float('Add. Exhibitors Price',digits=(15,2)),
        'order_add_guest': fields.integer('Additional Guests'),
        'price_add_guest': fields.float('Add. Guests Price',digits=(15,2)),
        'stand_reg_id': fields.many2one('event.registration','Registration'),
        'invoice_id': fields.many2one('account.invoice','Invoice'),
        'comments': fields.text('Comments'),
        'other_partner_ids': fields.text('Other Partners ID',help='List of id, separated by comma'),
        'video_status': fields.selection([('waiting','En attente'),('waiting_convert','A Convertir'),('ready','Ready')],'Video Status'),
        'video_name': fields.char('Video Name',size=1024),
        'video_price': fields.float('Supplement for Video Works',digits=(15,2)),
        'material_contact_id': fields.many2one('res.partner.contact','Material Contact'),
        'canvas_status': fields.selection([('waiting','En attente'),('waiting_convert','A retravailler'),('ready','Ready')],'Canvas Status'),
        'commissioned': fields.selection([('Thomas Schloesser','Thomas Schloesser'),('mailing','Mailings'),('other','Autre')],'Commission',required=True),
        'web_id': fields.integer('ID on Popup plateform'),
        'active': fields.boolean('Active'),
        'email_order': fields.function(_get_email_order,type='string',size=254,method=True,string='Email Contact Order'),
        'email_material': fields.function(_get_email_material,type='string',size=254,method=True,string='Email Contact Material'),
        'email_stand': fields.function(_get_email_stand,type='string',size=254,method=True,string='Email Stand Responsible'),
        'confirm_sending': fields.datetime('Confirmation Sending'),
        'count_value': fields.integer('Count as'),
        
        #'history_ids': fields.one2many('fullpage.history','fullpage_id','History'),
    }
    _defaults = {
        'video_status': lambda *a: 'waiting',
        'canvas_status' : lambda *a : 'waiting',
        'active': lambda *a: True,
        'price_add_exhibitor': lambda *a: 25.0,
        'price_add_guest': lambda *a: 25.0,
        'count_value': lambda *a: 1,
    }
cci_popup_stand()


